/* 
 *
 * Copyright (c) 2001-2002, 
 *  George C. Necula    <necula@cs.berkeley.edu>
 *  Scott McPeak        <smcpeak@cs.berkeley.edu>
 *  Wes Weimer          <weimer@cs.berkeley.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// sm: There's a subtle problem: stdio.h gets #included in some strange
// ways by some files.  For example, pwd.h says (line 69)
//   #define __need_FILE
//   #include <stdio.h>
// which causes stdio.h to provide the typedef for FILE, but omit
// everything else (like the #definition of EOF).  But then these wrappers
// get pulled in, expecting EOF, and we fail.  So I've now made this
// stuff conditional upon EOF being #defined, and hope that will ensure
// the code below is only seen during "normal" processing of stdio.h.
#if defined(DEPUTY) && defined(EOF) && ! defined(STDIO_ANNOT_INCLUDED)
#define STDIO_ANNOT_INCLUDED

// Unfortunately varargs depend on the compiler version
#ifndef _G_va_list
// This is needed for gcc 3.4.4
#define _G_va_list __gnuc_va_list
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern int (DPRINTF(2) fprintf)  (FILE * SAFE __stream,
		                  __const char * NTS __format, ...);
extern int (DPRINTF(1) printf)   (__const char * NTS __format, ...);
extern int (DPRINTF(2) sprintf)  (char * TRUSTED __s,
		                  __const char * NTS __format, ...);
extern int (DPRINTF(3) snprintf) (char * NT COUNT(__maxlen-1) __s,
                                  size_t __maxlen,
		                  __const char * NTS __restrict __format, ...);
extern int (DPRINTF(2) vfprintf) (FILE * SAFE __s, __const char * NTS __format,
		                  __gnuc_va_list __arg);
extern int (DPRINTF(1) vprintf)  (__const char * NTS __format,
                                   __gnuc_va_list __arg);
extern int (DPRINTF(2) vsprintf) (char * TRUSTED __s,
                                  __const char * NTS __format,
                                  __gnuc_va_list __arg);

extern int	fscanf(FILE * SAFE NONNULL, const char * NTS, ...);
extern int	scanf(const char * NTS, ...);
extern int	sscanf(const char * TRUSTED, const char * NTS, ...);

extern char * NTS 
   fgets (char *NT COUNT(__n-1) NONNULL __s, int __n, FILE * NONNULL __stream);

extern FILE * fdopen(int filedes, const char * NTS NONNULL mode);

extern int fputs(const char * NTS NONNULL s, FILE * SAFE NONNULL fl);

extern int puts(const char* NTS NONNULL s);
  

size_t fread (void * COUNT(_size * _n) NONNULL, size_t _size,
              size_t _n, FILE * SAFE NONNULL);
size_t fwrite (const void * COUNT(_size  *_n) NONNULL,
               size_t _size, size_t _n, FILE * SAFE NONNULL);


extern void perror(const char * NTS NONNULL); 

FILE * SAFE NULLABLE fopen (const char * NTS NONNULL _name,
                            const char * NTS NONNULL _type);

int fseek (FILE * SAFE NONNULL, long, int);

int remove (const char * NTS NONNULL);
int rename (const char * NTS NONNULL, const char * NTS NONNULL);
  
#ifdef __cplusplus
}
#endif // __cplusplus

#endif // DEPUTY
