/* 
 *
 * Copyright (c) 2001-2002, 
 *  George C. Necula    <necula@cs.berkeley.edu>
 *  Jeremy Condit       <jcondit@cs.berkeley.edu>
 *  Matt Harren         <matth@cs.berkeley.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


//wrappers for bzero, str*, and mem*

#if defined(DEPUTY) && ! defined(STRING_ANNOT_INCLUDED)
#define STRING_ANNOT_INCLUDED

  #ifdef __cplusplus
     extern "C" {
  #endif

#include "string_macros.h"

       // If gcc is in optimize mode, it turns many string functions into
       // ugly messes, which we cannot deal with
#ifdef strchr
#undef strchr
#undef strcmp
#undef strncmp       
#undef strdup
#undef strpbrk
#undef strsep       
#undef strspn      
#undef strcspn
#undef strncat
#undef strncpy
#endif
       
unsigned int  strlen(const STRING s);

void * (DMEMSET(1, 2, 3) memset)(void* p, int what, size_t sz);
int    (DMEMCMP(1, 2, 3) memcmp)(void* s1, void* s2, size_t sz);
void * (DMEMCPY(1, 2, 3) memcpy)(void* dst, void* src, size_t sz);
void * (DMEMCPY(1, 2, 3) memmove)(void *dst, void* src, size_t sz);

// void bzero (char * COUNT(size) buff, unsigned int size);

// strcpy is not safe. Do not use       

// strncpy copies at most n characters, including the 0, rest is filled with 0
       
STRING strncpy(STRINGBUF(n) dest,
               const STRING src, size_t n);
STRING __builtin_strncpy(STRINGBUF(n) dest,
                         const STRING src, size_t n);
//dinfer.ml may change some strcpy to strncpy.
#pragma cilnoremove("strncpy")

int  strcmp(const STRING s1,
            const STRING s2);
int  strncmp(const STRING s1,
             const STRING s2, size_t n);
     
size_t strlcpy(STRINGBUF(siz-1) dst,
               const STRING src,
               size_t siz);

// strcat is unsafe. Use strncat
// For strncat, n is the number of characters to copy !!!
STRING strncat(STRINGBUF(n) dest, const STRING src, size_t n);
STRING __builtin_strncat(STRINGBUF(n) dest,
                         const STRING src, size_t n);

size_t strlcat(STRINGBUF(n-1) dest, const STRING src, size_t n);

       
OPTSTRING strchr(const STRING s, int chr);
OPTSTRING strrchr(const STRING s, int chr);
OPTSTRING strdup(const STRING s);
OPTSTRING strpbrk(const STRING str, const STRING accept_arg);
OPTSTRING strsep(char * NTS * NT stringp, const STRING delim);

size_t strspn(const STRING str, const STRING charset);
size_t strcspn(const STRING str, const STRING charset);

int strcasecmp(const STRING s1, const STRING s2);
int strncasecmp(const STRING s1, const STRING s2, size_t n);

OPTSTRING strtok(OPTSTRING str, const STRING delim);
//strtok_r is a macro
//OPTSTRING strtok_r(OPTSTRING str, const STRING delim, STRING * SAFE ptrptr);

OPTSTRING strerror(int errnum);
OPTSTRING strstr(const STRING __haystack,
                 const STRING __needle);

size_t strspn(const STRING s, const STRING accept);
       
  #ifdef __cplusplus
     }
  #endif
#endif // DEPUTY









