(*
 *
 * Copyright (c) 2004, 
 *  Jeremy Condit       <jcondit@cs.berkeley.edu>
 *  George C. Necula    <necula@cs.berkeley.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

module C = Cil
module RD = Reachingdefs
module AE = Availexps
module Z = Zrapp

let debug : bool ref = ref false
let verbose : bool ref = ref false
let trustAll : bool ref = ref false
let stats: bool ref = ref false
let optLevel : int ref = ref 3
let parseFile : string ref = ref ""
let inferFile : string ref = ref ""
let assumeString : bool ref = ref false
let alwaysStopOnError : bool ref = ref false
let fastChecks : bool ref = ref false
let multipleErrors : bool ref = ref false
let inferKinds: bool ref = ref true
let warnAsm: bool ref = ref false

(** How much detail to print.  -1 means do not print the graph *)
let emitGraphDetailLevel : int ref = ref (-1)


let options =
[ (* General *)
  "", Arg.Unit (fun () -> ()), "\n\n\t\tGeneral:\n";
  "--deputyverbose", Arg.Set verbose,
       "Enable verbose output for Deputy";
  "--deputystats", Arg.Set stats,
       "output optimizer runtime stats";

  (* Preprocessing and inference *)
  "", Arg.Unit (fun () -> ()), "\n\n\t\tPreprocessing:\n";
  "--deputyparseout", Arg.Set_string parseFile,
        "File in which to place Deputy parsing results, before preprocessing";
  "--deputynoinfer", Arg.Clear inferKinds,
        "Don't use CCured-style interprocedural analysis to determine kinds for unannotated pointers.";
  "--deputyinferout", Arg.Set_string inferFile,
       ("File in which to place the results of Deputy's preprocessing,\n\t\t"^
        "   including inference results.");
  "--deputyinferoutdetail", Arg.Set_int emitGraphDetailLevel,
        ("\008=n Dump the inference graph with the specified level\n\t\t\t   "^
       "of detail, with n=0 being the most terse and n=3 the most\n\t\t\t   "^
       "verbose.  Has no effect unless --deputyinferout\n\t\t\t   "^
       "and the interprocedural inference are both used.");

  (* Typechecking *)
  "", Arg.Unit (fun () -> ()), "\n\n\t\tTypechecking:\n";
  "--deputyMultipleErrors", Arg.Set multipleErrors,
       "Attempt to continue processing on error";
  "--deputytrust", Arg.Set trustAll,
        "Trust all bad casts by default";
  "--deputyassumestring", Arg.Set assumeString,
        "Assume all char arrays, and all unannotated char*s in function types, are NT.";
  "--deputyWarnAsm",
       Arg.Set warnAsm, "Show all warnings when assembly is ignored";

  (* Code gen and optimizations *)
  "", Arg.Unit (fun () -> ()), "\n\n\t\tCodegen:\n";
  "--deputyopt", Arg.Set_int optLevel,
       ("Control deputy optimizations:\n\t\t" ^
        "0: no optimization\n\t\t" ^
        "1: flow-insensitive optimization\n\t\t" ^
        "2: some flow-sensitive optimization\n\t\t"^
        "3: all optimizations(Default)");
  "--deputyAlwaysStopOnError", Arg.Set alwaysStopOnError,
       "Optimize checks assuming that we stop on error";
  "--deputyFastChecks", Arg.Set fastChecks,
       "Optimize checks assuming that we stop on error without printing specifics about the failure";
  "--deputyZrapp", 
       Arg.Unit (fun n -> C.lineDirectiveStyle := None;
         C.printerForMaincil := Z.zraCilPrinter),
       "Use zrapp";

  (* Things end users usually won't need *)
  "", Arg.Unit (fun () -> ()), "\n\n\t\tAdvanced (for debugging Deputy):\n";
  "--deputyOptimDebug", 
       Arg.Unit (fun n ->  Z.debug := true; 
         RD.debug := true; AE.debug := true; debug := true),
       "Have the optimizer output lots of debugging info";
  ]
